@@ -34,13 +34,7 @@
 #include <xfce-mcs-manager/manager-plugin.h>
 #include <libxfcegui4/libxfcegui4.h>
 
-#define RCDIR "mcs_settings"
-#define OLDRCDIR "settings"
-#define CHANNEL1 "settings"
-#define RCFILE1 "gtk.xml"
-#define CHANNEL2 "mouse"
-#define RCFILE2 "mouse.xml"
-#define PLUGIN_NAME "mouse"
+#include "mouse-plugin-internal.h"
 
 #define DEFAULT_ACCELERATION    2
 #define DEFAULT_THRESHOLD       4
@@ -53,14 +47,13 @@
 #define THRESH_MIN      1
 #define THRESH_MAX      20
 #define DND_THRESH_MIN  1
-#define DND_THRESH_MAX  20
+#define DND_THRESH_MAX  50
 #define DBL_CLICK_MIN   100
 #define DBL_CLICK_MAX   2000
 #define DEFAULT_ICON_SIZE 48
 #define DEFAULT_PTR_MAP_SIZE 128
 
 static void create_channel(McsPlugin * mcs_plugin);
-static gboolean write_options(McsPlugin * mcs_plugin);
 static void run_dialog(McsPlugin * mcs_plugin);
 
 static gboolean is_running = FALSE;
@@ -72,51 +65,6 @@ static int threshold = DEFAULT_THRESHOLD;
 static int denominator = DEFAULT_DENOMINATOR;
 
 
-typedef struct _Itf Itf;
-struct _Itf
-{
-    McsPlugin *mcs_plugin;
-
-    GSList *radiobutton1_group;
-
-    GtkWidget *mouse_dialog;
-    GtkWidget *dialog_vbox1;
-    GtkWidget *dialog_header;
-    GtkWidget *vbox1;
-    GtkWidget *frame1;
-    GtkWidget *hbox2;
-    GtkWidget *radiobutton1;
-    GtkWidget *radiobutton2;
-    GtkWidget *label1;
-    GtkWidget *frame2;
-    GtkWidget *table1;
-    GtkWidget *label3;
-    GtkWidget *label4;
-    GtkWidget *hscale1;
-    GtkWidget *label5;
-    GtkWidget *label6;
-    GtkWidget *label7;
-    GtkWidget *hscale2;
-    GtkWidget *label8;
-    GtkWidget *label2;
-    GtkWidget *frame3;
-    GtkWidget *table2;
-    GtkWidget *label11;
-    GtkWidget *hscale3;
-    GtkWidget *label12;
-    GtkWidget *label10;
-    GtkWidget *label9;
-    GtkWidget *frame4;
-    GtkWidget *table3;
-    GtkWidget *label14;
-    GtkWidget *label15;
-    GtkWidget *label16;
-    GtkWidget *hscale4;
-    GtkWidget *label13;
-    GtkWidget *dialog_action_area1;
-    GtkWidget *closebutton1;
-};
-
 static void get_mouse_values(int *accel_return, int *denom_return, int *thresh_return)
 {
     gdk_flush();
@@ -201,7 +149,7 @@ static void cb_righthandled_changed(GtkWidget * dialog, gpointer user_data)
 
     mcs_manager_set_int(mcs_plugin->manager, "Mouse/RightHanded", CHANNEL2, right_handed ? 1 : 0);
     mcs_manager_notify(mcs_plugin->manager, CHANNEL2);
-    write_options(mcs_plugin);
+    mouse_plugin_write_options(mcs_plugin);
 }
 
 static void cb_acceleration_changed(GtkWidget * dialog, gpointer user_data)
@@ -215,7 +163,7 @@ static void cb_acceleration_changed(GtkWidget * dialog, gpointer user_data)
     mcs_manager_set_int(mcs_plugin->manager, "Mouse/Acceleration", CHANNEL2, acceleration);
 
     mcs_manager_notify(mcs_plugin->manager, CHANNEL2);
-    write_options(mcs_plugin);
+    mouse_plugin_write_options(mcs_plugin);
 }
 
 static void cb_threshold_changed(GtkWidget * dialog, gpointer user_data)
@@ -229,7 +177,7 @@ static void cb_threshold_changed(GtkWidget * dialog, gpointer user_data)
     mcs_manager_set_int(mcs_plugin->manager, "Mouse/Threshold", CHANNEL2, threshold);
 
     mcs_manager_notify(mcs_plugin->manager, CHANNEL2);
-    write_options(mcs_plugin);
+    mouse_plugin_write_options(mcs_plugin);
 }
 
 static void cb_dnd_threshold_changed(GtkWidget * dialog, gpointer user_data)
@@ -242,7 +190,7 @@ static void cb_dnd_threshold_changed(GtkWidget * dialog, gpointer user_data)
     mcs_manager_set_int(mcs_plugin->manager, "Net/DndDragThreshold", CHANNEL1, dnd_threshold);
 
     mcs_manager_notify(mcs_plugin->manager, CHANNEL1);
-    write_options(mcs_plugin);
+    mouse_plugin_write_options(mcs_plugin);
 }
 
 static void cb_dbl_clicktime_changed(GtkWidget * dialog, gpointer user_data)
@@ -255,13 +203,17 @@ static void cb_dbl_clicktime_changed(GtkWidget * dialog, gpointer user_data)
     mcs_manager_set_int(mcs_plugin->manager, "Net/DoubleClickTime", CHANNEL1, dbl_clicktime);
 
     mcs_manager_notify(mcs_plugin->manager, CHANNEL1);
-    write_options(mcs_plugin);
+    mouse_plugin_write_options(mcs_plugin);
 }
 
 Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
 {
     Itf *dialog;
     GtkSizeGroup *sg;
+    GtkWidget *frame_bin;
+#ifdef HAVE_XCURSOR_EXTENSION
+    GtkWidget *lbl;
+#endif
 
     dialog = g_new(Itf, 1);
 
@@ -282,21 +234,37 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
                                                _("Mouse Preferences"));
     gtk_widget_show(dialog->dialog_header);
     gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->dialog_header, FALSE, TRUE, 0);
+    
+    dialog->notebook_vbox = gtk_vbox_new(FALSE, 6);
+    gtk_container_set_border_width(GTK_CONTAINER(dialog->notebook_vbox), 6);
+    gtk_widget_show(dialog->notebook_vbox);
+    gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->notebook_vbox, TRUE, TRUE, 0);
 
+#ifdef HAVE_XCURSOR_EXTENSION
+    dialog->notebook = gtk_notebook_new();
+    gtk_widget_show(dialog->notebook);
+    gtk_box_pack_start(GTK_BOX(dialog->notebook_vbox), dialog->notebook, TRUE, TRUE, 0);
+    
+    lbl = gtk_label_new(_("Behavior"));
+    gtk_widget_show(lbl);
     dialog->vbox1 = gtk_vbox_new(FALSE, 6);
+    gtk_container_set_border_width(GTK_CONTAINER(dialog->vbox1), 6);
     gtk_widget_show(dialog->vbox1);
-    gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->vbox1, TRUE, TRUE, 6);
+    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->notebook), dialog->vbox1, lbl);
+#else
+    dialog->vbox1 = dialog->notebook_vbox;
+#endif
 
     sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
     
     /* button order */
-    dialog->frame1 = xfce_framebox_new (_("Button settings"), TRUE);
+    dialog->frame1 = xfce_create_framebox (_("Button settings"), &frame_bin);
     gtk_widget_show(dialog->frame1);
     gtk_box_pack_start(GTK_BOX(dialog->vbox1), dialog->frame1, FALSE, TRUE, 0);
 
     dialog->hbox2 = gtk_hbox_new(FALSE, 0);
     gtk_widget_show(dialog->hbox2);
-    xfce_framebox_add (XFCE_FRAMEBOX (dialog->frame1), dialog->hbox2);
+    gtk_container_add (GTK_CONTAINER (frame_bin), dialog->hbox2);
     
     dialog->radiobutton1 = gtk_radio_button_new_with_mnemonic(NULL, _("Left Handed"));
     gtk_widget_show(dialog->radiobutton1);
@@ -313,13 +281,13 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->radiobutton2), right_handed);
 
     /* motion settings */
-    dialog->frame2 = xfce_framebox_new (_("Motion settings"), TRUE);
+    dialog->frame2 = xfce_create_framebox (_("Motion settings"), &frame_bin);
     gtk_widget_show(dialog->frame2);
     gtk_box_pack_start(GTK_BOX(dialog->vbox1), dialog->frame2, FALSE, TRUE, 0);
 
     dialog->table1 = gtk_table_new(2, 4, FALSE);
     gtk_widget_show(dialog->table1);
-    xfce_framebox_add (XFCE_FRAMEBOX (dialog->frame2), dialog->table1);
+    gtk_container_add (GTK_CONTAINER (frame_bin), dialog->table1);
 
     dialog->label3 = gtk_label_new(_("Acceleration :"));
     gtk_widget_show(dialog->label3);
@@ -374,13 +342,13 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     gtk_misc_set_alignment(GTK_MISC(dialog->label8), 0, 0.5);
 
     /* drag and drop */
-    dialog->frame3 = xfce_framebox_new (_("Drag and drop"), TRUE);
+    dialog->frame3 = xfce_create_framebox (_("Drag and drop"), &frame_bin);
     gtk_widget_show(dialog->frame3);
     gtk_box_pack_start(GTK_BOX(dialog->vbox1), dialog->frame3, FALSE, TRUE, 0);
 
     dialog->table2 = gtk_table_new(1, 4, FALSE);
     gtk_widget_show(dialog->table2);
-    xfce_framebox_add (XFCE_FRAMEBOX (dialog->frame3), dialog->table2);
+    gtk_container_add (GTK_CONTAINER (frame_bin), dialog->table2);
 
     dialog->label10 = gtk_label_new(_("Threshold :"));
     gtk_widget_show(dialog->label10);
@@ -409,13 +377,13 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     gtk_misc_set_alignment(GTK_MISC(dialog->label12), 0, 0.5);
 
     /* double click */
-    dialog->frame4 = xfce_framebox_new (_("Double click"), TRUE);
+    dialog->frame4 = xfce_create_framebox (_("Double click"), &frame_bin);
     gtk_widget_show(dialog->frame4);
     gtk_box_pack_start(GTK_BOX(dialog->vbox1), dialog->frame4, FALSE, TRUE, 0);
 
     dialog->table3 = gtk_table_new(1, 4, FALSE);
     gtk_widget_show(dialog->table3);
-    xfce_framebox_add (XFCE_FRAMEBOX (dialog->frame4), dialog->table3);
+    gtk_container_add (GTK_CONTAINER (frame_bin), dialog->table3);
 
     dialog->label14 = gtk_label_new(_("Speed :"));
     gtk_widget_show(dialog->label14);
@@ -454,7 +422,12 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
     gtk_dialog_add_action_widget(GTK_DIALOG(dialog->mouse_dialog), dialog->closebutton1, GTK_RESPONSE_CLOSE);
     GTK_WIDGET_SET_FLAGS(dialog->closebutton1, GTK_CAN_DEFAULT);
 
-    gtk_widget_set_size_request (dialog->mouse_dialog, 300, -1);
+#ifdef HAVE_XCURSOR_EXTENSION
+    lbl = gtk_label_new(_("Cursor"));
+    gtk_widget_show(lbl);
+    mouse_plugin_create_cursor_page(dialog);
+    gtk_notebook_append_page(GTK_NOTEBOOK(dialog->notebook), dialog->cursor_page, lbl);
+#endif
     
     return dialog;
 }
@@ -471,6 +444,8 @@ static void setup_dialog(Itf * itf)
     g_signal_connect(G_OBJECT(itf->hscale4), "value_changed", (GCallback) cb_dbl_clicktime_changed, itf);
 
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (itf->mouse_dialog));
+    gdk_x11_window_set_user_time(GTK_WIDGET (itf->mouse_dialog)->window, 
+            gdk_x11_get_server_time (GTK_WIDGET (itf->mouse_dialog)->window));
     gtk_widget_show(itf->mouse_dialog);
 }
 
@@ -554,7 +529,7 @@ create_channel(McsPlugin *mcs_plugin)
     setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Net/DndDragThreshold", CHANNEL1);
     if(setting)
     {
-        dnd_threshold = (setting->data.v_int ? TRUE : FALSE);
+        dnd_threshold = setting->data.v_int;
     }
     else
     {
@@ -607,11 +582,18 @@ create_channel(McsPlugin *mcs_plugin)
 
     set_mouse_values(right_handed, acceleration, threshold);
     
-    write_options (mcs_plugin);
+#ifdef HAVE_XCURSOR_EXTENSION
+    mouse_plugin_set_initial_cursor_values(mcs_plugin);
+#endif    
+
+#if 0    
+    /* I fail to see why we need to save the options here, during startup... */
+    mouse_plugin_write_options (mcs_plugin);
+#endif
 }
 
-static gboolean
-write_options(McsPlugin *mcs_plugin)
+gboolean
+mouse_plugin_write_options(McsPlugin *mcs_plugin)
 {
     gboolean result = FALSE;
     gchar *rcfile, *path;
@@ -640,16 +622,22 @@ write_options(McsPlugin *mcs_plugin)
 static void
 run_dialog(McsPlugin *mcs_plugin)
 {
-    Itf *dialog;
+    static Itf *dialog = NULL;
 
-    if (!is_running) {
-        is_running = TRUE;
+    xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
-        xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
-
-        dialog = create_mouse_dialog(mcs_plugin);
-        setup_dialog(dialog);
+    if (is_running)
+    {
+        if((dialog) && (dialog->mouse_dialog))
+        {
+            gtk_window_present(GTK_WINDOW(dialog->mouse_dialog));
+        }
+        return;
     }
+
+    is_running = TRUE;
+    dialog = create_mouse_dialog(mcs_plugin);
+    setup_dialog(dialog);
 }
 
 /* macro defined in manager-plugin.h */
