@@ -133,6 +133,7 @@ static const char *aa_vbgr_xpm[] = {
 static gboolean setting_model = FALSE;
 static gboolean setting_itheme_model = FALSE;
 static gboolean is_running = FALSE;
+static gboolean can_change_accel = FALSE;
 static gchar *current_theme = NULL;
 static gchar *current_icon_theme = NULL;
 static gchar *current_font = NULL;
@@ -169,6 +170,8 @@ struct _Itf
     GtkWidget *button3;
     GtkWidget *frame3;
     GtkWidget *optionmenu1;
+    GtkWidget *frame5;
+    GtkWidget *accel_checkbox;
     GtkWidget *menu1;
     GtkWidget *menuitem1;
     GtkWidget *menuitem2;
@@ -604,6 +607,20 @@ static void on_both_horiz_activate(GtkMenuItem * menuitem, gpointer user_data)
 }
 
 
+static void on_change_accel_toggled(GtkToggleButton * button, gpointer user_data)
+{
+    Itf *itf = (Itf *) user_data;
+    McsPlugin *mcs_plugin = itf->mcs_plugin;
+    gboolean active = gtk_toggle_button_get_active(button);
+
+    can_change_accel = (active) ? 1 : 0;
+    mcs_manager_set_int(mcs_plugin->manager, "Gtk/CanChangeAccels", CHANNEL, can_change_accel);
+    mcs_manager_notify(mcs_plugin->manager, CHANNEL);
+    write_options(mcs_plugin);
+    apply_xft_options(itf);
+}
+
+
 static void on_antialiasing_toggled(GtkToggleButton * button, gpointer user_data)
 {
     Itf *itf = (Itf *) user_data;
@@ -920,21 +937,22 @@ Itf *create_theme_dialog(McsPlugin * mcs_plugin)
     gtk_option_menu_set_menu(GTK_OPTION_MENU(dialog->optionmenu1), dialog->menu1);
 
     if(!strcmp(current_toolbar_style, "text"))
-    {
         gtk_option_menu_set_history(GTK_OPTION_MENU(dialog->optionmenu1), 1);
-    }
     else if(!strcmp(current_toolbar_style, "both"))
-    {
         gtk_option_menu_set_history(GTK_OPTION_MENU(dialog->optionmenu1), 2);
-    }
     else if(!strcmp(current_toolbar_style, "both-horiz"))
-    {
         gtk_option_menu_set_history(GTK_OPTION_MENU(dialog->optionmenu1), 3);
-    }
     else
-    {
         gtk_option_menu_set_history(GTK_OPTION_MENU(dialog->optionmenu1), 0);
-    }
+
+    dialog->frame5 = xfce_framebox_new(_("Menu Accelerators"), TRUE);
+    gtk_widget_show(dialog->frame5);
+    gtk_box_pack_start(GTK_BOX(dialog->vbox3), dialog->frame5, TRUE, FALSE, 0);
+
+    dialog->accel_checkbox = gtk_check_button_new_with_label(_("Editable menu accelerators"));
+    gtk_widget_show(dialog->accel_checkbox);
+    xfce_framebox_add(XFCE_FRAMEBOX(dialog->frame5), dialog->accel_checkbox);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->accel_checkbox), (gboolean) can_change_accel);    
 
     /**
      ** XXX move this to a separate notebook tab?
@@ -1113,6 +1131,7 @@ static void setup_dialog(Itf * itf)
     g_signal_connect(G_OBJECT(itf->menuitem2), "activate", G_CALLBACK(on_text_activate), itf);
     g_signal_connect(G_OBJECT(itf->menuitem3), "activate", G_CALLBACK(on_both_activate), itf);
     g_signal_connect(G_OBJECT(itf->menuitem4), "activate", G_CALLBACK(on_both_horiz_activate), itf);
+    g_signal_connect(G_OBJECT(itf->accel_checkbox), "toggled", G_CALLBACK(on_change_accel_toggled), itf);
 
     g_signal_connect(G_OBJECT(itf->aa_checkbox1), "toggled", G_CALLBACK(on_antialiasing_toggled), itf);
     g_signal_connect(G_OBJECT(itf->aa_checkbox2), "toggled", G_CALLBACK(on_hinting_toggled), itf);
@@ -1127,6 +1146,8 @@ static void setup_dialog(Itf * itf)
     g_signal_connect(G_OBJECT(itf->aa_item24), "activate", G_CALLBACK(on_rgba_vbgr_activate), itf);
 
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (itf->theme_dialog));
+    gdk_x11_window_set_user_time(GTK_WIDGET (itf->theme_dialog)->window, 
+            gdk_x11_get_server_time (GTK_WIDGET (itf->theme_dialog)->window));
     gtk_widget_show(itf->theme_dialog);
 }
 
@@ -1249,6 +1270,16 @@ static void create_channel(McsPlugin * mcs_plugin)
         mcs_manager_set_string(mcs_plugin->manager, "Gtk/ToolbarStyle", CHANNEL, current_toolbar_style);
     }
 
+    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Gtk/CanChangeAccels", CHANNEL);
+    if(setting)
+    {
+        can_change_accel = (gboolean) setting->data.v_int;
+    }
+    else
+    {
+        mcs_manager_set_int(mcs_plugin->manager, "Gtk/CanChangeAccels", CHANNEL, can_change_accel);
+    }
+
     setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Xft/Antialias", CHANNEL);
     if(setting)
     {
@@ -1256,7 +1287,6 @@ static void create_channel(McsPlugin * mcs_plugin)
     }
     else
     {
-        current_xft_antialias = DEFAULT_XFT_ANTIALIAS;
         mcs_manager_set_int(mcs_plugin->manager, "Xft/Antialias", CHANNEL, current_xft_antialias);
     }
 
@@ -1267,7 +1297,6 @@ static void create_channel(McsPlugin * mcs_plugin)
     }
     else
     {
-        current_xft_hinting = DEFAULT_XFT_HINTING;
         mcs_manager_set_int(mcs_plugin->manager, "Xft/Hinting", CHANNEL, current_xft_hinting);
     }
 
@@ -1301,7 +1330,10 @@ static void create_channel(McsPlugin * mcs_plugin)
         mcs_manager_set_string(mcs_plugin->manager, "Xft/RGBA", CHANNEL, current_xft_rgba);
     }
     
+#if 0    
+    /* I fail to see why we need to save the options here, during startup... */
     write_options (mcs_plugin);
+#endif
 }
 
 static gboolean write_options(McsPlugin * mcs_plugin)
@@ -1361,20 +1393,26 @@ static void apply_xft_options(Itf * itf)
 
 static void run_dialog(McsPlugin * mcs_plugin)
 {
-    Itf *dialog;
+    static Itf *dialog = NULL;
+
+    xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
     if(tooltips == NULL)
         tooltips = gtk_tooltips_new();
 
-    if(!is_running)
+    if (is_running)
     {
-        is_running = TRUE;
-
-        xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
-
-        dialog = create_theme_dialog(mcs_plugin);
-        setup_dialog(dialog);
+        if((dialog) && (dialog->theme_dialog))
+        {
+            gtk_window_present(GTK_WINDOW(dialog->theme_dialog));
+            gtk_window_set_focus (GTK_WINDOW(dialog->theme_dialog), NULL);
+        }
+        return;
     }
+
+    is_running = TRUE;
+    dialog = create_theme_dialog(mcs_plugin);
+    setup_dialog(dialog);
 }
 
 /* macro defined in manager-plugin.h */
